@@ -3,6 +3,7 @@ import fnmatch
 import logging
 import os
 import re
+import warnings
 from copy import deepcopy
 from pprint import pformat
 
@@ -14,9 +15,9 @@ from . import _recipe_checks as check
 from ._config import (
     TAGS,
     get_activity,
+    get_extra_facets,
     get_institutes,
     get_project_config,
-    replace_tags,
 )
 from ._data_finder import (
     get_input_filelist,
@@ -40,6 +41,7 @@ from .preprocessor._derive import get_required
 from .preprocessor._download import synda_search
 from .preprocessor._io import DATASET_KEYS, concatenate_callback
 from .preprocessor._regrid import (
+    _spec_to_latlonvals,
     get_cmor_levels,
     get_reference_levels,
     parse_cell_spec,
@@ -97,6 +99,13 @@ def _add_cmor_info(variable, override=False):
     check.variable(variable, required_keys=cmor_keys)
 
 
+def _add_extra_facets(variable, extra_facets_dir):
+    extra_facets = get_extra_facets(variable["project"], variable["dataset"],
+                                    variable["mip"], variable["short_name"],
+                                    extra_facets_dir)
+    _augment(variable, extra_facets)
+
+
 def _special_name_to_dataset(variable, special_name):
     """Convert special names to dataset names."""
     if special_name in ('reference_dataset', 'alternative_dataset'):
@@ -141,6 +150,7 @@ def _update_target_levels(variable, variables, settings, config_user):
         else:
             variable_data = _get_dataset_info(dataset, variables)
             filename = _dataset_to_file(variable_data, config_user)
+            fix_dir = f"{os.path.splitext(variable_data['filename'])[0]}_fixed"
             settings['extract_levels']['levels'] = get_reference_levels(
                 filename=filename,
                 project=variable_data['project'],
@@ -148,8 +158,7 @@ def _update_target_levels(variable, variables, settings, config_user):
                 short_name=variable_data['short_name'],
                 mip=variable_data['mip'],
                 frequency=variable_data['frequency'],
-                fix_dir=os.path.splitext(variable_data['filename'])[0] +
-                '_fixed',
+                fix_dir=fix_dir,
             )
 
 
@@ -168,7 +177,12 @@ def _update_target_grid(variable, variables, settings, config_user):
             _get_dataset_info(grid, variables), config_user)
     else:
         # Check that MxN grid spec is correct
-        parse_cell_spec(settings['regrid']['target_grid'])
+        target_grid = settings['regrid']['target_grid']
+        if isinstance(target_grid, str):
+            parse_cell_spec(target_grid)
+        # Check that cdo spec is correct
+        elif isinstance(target_grid, dict):
+            _spec_to_latlonvals(**target_grid)
 
 
 def _update_regrid_time(variable, settings):
@@ -260,12 +274,7 @@ def _get_default_settings(variable, config_user, derive=False):
     settings['concatenate'] = {}
 
     # Configure fixes
-    fix = {
-        'project': variable['project'],
-        'dataset': variable['dataset'],
-        'short_name': variable['short_name'],
-        'mip': variable['mip'],
-    }
+    fix = deepcopy(variable)
     # File fixes
     fix_dir = os.path.splitext(variable['filename'])[0] + '_fixed'
     settings['fix_file'] = dict(fix)
@@ -314,16 +323,21 @@ def _get_default_settings(variable, config_user, derive=False):
     if variable['short_name'] != variable['original_short_name']:
         settings['save']['alias'] = variable['short_name']
 
+    # Configure fx settings
+    settings['add_fx_variables'] = {
+        'fx_variables': {},
+        'check_level': config_user.get('check_level', CheckLevels.DEFAULT)
+    }
+    settings['remove_fx_variables'] = {}
+
     return settings
 
 
-def _add_fxvar_keys(fx_var_dict, variable):
+def _add_fxvar_keys(fx_info, variable):
     """Add keys specific to fx variable to use get_input_filelist."""
-    fx_variable = dict(variable)
-    fx_variable.update(fx_var_dict)
-
-    # set variable names
-    fx_variable['variable_group'] = fx_var_dict['short_name']
+    fx_variable = deepcopy(variable)
+    fx_variable.update(fx_info)
+    fx_variable['variable_group'] = fx_info['short_name']
 
     # add special ensemble for CMIP5 only
     if fx_variable['project'] == 'CMIP5':
@@ -335,77 +349,67 @@ def _add_fxvar_keys(fx_var_dict, variable):
     return fx_variable
 
 
-def _get_fx_file(variable, fx_variable, config_user):
+def _search_fx_mip(tables, found_mip, variable, fx_info, config_user):
+    fx_files = None
+    for mip in tables:
+        fx_cmor = tables[mip].get(fx_info['short_name'])
+        if fx_cmor:
+            found_mip = True
+            fx_info['mip'] = mip
+            fx_info = _add_fxvar_keys(fx_info, variable)
+            logger.debug("For fx variable '%s', found table '%s'",
+                         fx_info['short_name'], mip)
+            fx_files = _get_input_files(fx_info, config_user)[0]
+            if fx_files:
+                logger.debug("Found fx variables '%s':\n%s",
+                             fx_info['short_name'], pformat(fx_files))
+    return found_mip, fx_info, fx_files
+
+
+def _get_fx_files(variable, fx_info, config_user):
     """Get fx files (searching all possible mips)."""
-    # make it a dict
-    if isinstance(fx_variable, str):
-        fx_varname = fx_variable
-        fx_variable = {'short_name': fx_varname}
-    else:
-        fx_varname = fx_variable['short_name']
 
     # assemble info from master variable
-    var = dict(variable)
     var_project = variable['project']
     # check if project in config-developer
     try:
         get_project_config(var_project)
     except ValueError:
-        raise RecipeError(
-            f"Requested fx variable '{fx_varname}' with parent variable"
-            f"'{variable}' does not have a '{var_project}' project"
-            f"in config-developer.")
-    cmor_table = CMOR_TABLES[var_project]
-    valid_fx_vars = []
+        raise RecipeError(f"Requested fx variable '{fx_info['short_name']}' "
+                          f"with parent variable '{variable}' does not have "
+                          f"a '{var_project}' project in config-developer.")
+    project_tables = CMOR_TABLES[var_project].tables
 
     # force only the mip declared by user
-    if 'mip' in fx_variable:
-        fx_mips = [fx_variable['mip']]
+    found_mip = False
+    if not fx_info['mip']:
+        found_mip, fx_info, fx_files = _search_fx_mip(project_tables,
+                                                      found_mip, variable,
+                                                      fx_info, config_user)
     else:
-        # Get all fx-related mips (original var mip,
-        # 'fx' and extend from cmor tables)
-        fx_mips = [variable['mip']]
-        fx_mips.extend(mip for mip in cmor_table.tables if 'fx' in mip)
-
-    # Search all mips for available variables
-    # priority goes to user specified mip if available
-    searched_mips = []
-    fx_files = []
-    for fx_mip in fx_mips:
-        fx_cmor_variable = cmor_table.get_variable(fx_mip, fx_varname)
-        if fx_cmor_variable is not None:
-            fx_var_dict = dict(fx_variable)
-            searched_mips.append(fx_mip)
-            fx_var_dict['mip'] = fx_mip
-            fx_var_dict = _add_fxvar_keys(fx_var_dict, var)
-            valid_fx_vars.append(fx_var_dict)
-            logger.debug("For fx variable '%s', found table '%s'", fx_varname,
-                         fx_mip)
-            fx_files = _get_input_files(fx_var_dict, config_user)[0]
-
-            # If files found, return them
-            if fx_files:
-                logger.debug("Found fx variables '%s':\n%s", fx_varname,
-                             pformat(fx_files))
-                break
+        fx_cmor = project_tables[fx_info['mip']].get(fx_info['short_name'])
+        if fx_cmor:
+            found_mip = True
+            fx_info = _add_fxvar_keys(fx_info, variable)
+            fx_files = _get_input_files(fx_info, config_user)[0]
 
     # If fx variable was not found in any table, raise exception
-    if not searched_mips:
+    if not found_mip:
         raise RecipeError(
-            f"Requested fx variable '{fx_varname}' not available in "
-            f"any 'fx'-related CMOR table ({fx_mips}) for '{var_project}'")
+            f"Requested fx variable '{fx_info['short_name']}' "
+            f"not available in any CMOR table for '{var_project}'")
 
     # flag a warning
     if not fx_files:
-        logger.warning("Missing data for fx variable '%s'", fx_varname)
+        logger.warning("Missing data for fx variable '%s'",
+                       fx_info['short_name'])
 
     # allow for empty lists corrected for by NE masks
     if fx_files:
-        fx_files = fx_files[0]
-    if valid_fx_vars:
-        valid_fx_vars = valid_fx_vars[0]
+        if fx_info['frequency'] == 'fx':
+            fx_files = fx_files[0]
 
-    return fx_files, valid_fx_vars
+    return fx_files, fx_info
 
 
 def _exclude_dataset(settings, variable, step):
@@ -431,14 +435,37 @@ def _update_fx_files(step_name, settings, variable, config_user, fx_vars):
     """Update settings with mask fx file list or dict."""
     if not fx_vars:
         return
+    for fx_var, fx_info in fx_vars.items():
+        if not fx_info:
+            fx_info = {}
+        if 'mip' not in fx_info:
+            fx_info.update({'mip': None})
+        if 'short_name' not in fx_info:
+            fx_info.update({'short_name': fx_var})
+        fx_files, fx_info = _get_fx_files(variable, fx_info, config_user)
+        if fx_files:
+            fx_info['filename'] = fx_files
+            settings['add_fx_variables']['fx_variables'].update({
+                fx_var: fx_info
+            })
+            logger.info(
+                'Using fx files for variable %s during step %s: %s',
+                variable['short_name'], step_name, pformat(fx_files))
 
-    fx_vars = [_get_fx_file(variable, fxvar, config_user) for fxvar in fx_vars]
 
-    fx_dict = {fx_var[1]['short_name']: fx_var[0] for fx_var in fx_vars}
-    settings['fx_variables'] = fx_dict
-    logger.info('Using fx_files: %s for variable %s during step %s',
-                pformat(settings['fx_variables']), variable['short_name'],
-                step_name)
+def _fx_list_to_dict(fx_vars):
+    """Convert fx list to dictionary.
+
+    To be deprecated at some point.
+    """
+    user_fx_vars = {}
+    for fx_var in fx_vars:
+        if isinstance(fx_var, dict):
+            short_name = fx_var['short_name']
+            user_fx_vars.update({short_name: fx_var})
+            continue
+        user_fx_vars.update({fx_var: None})
+    return user_fx_vars
 
 
 def _update_fx_settings(settings, variable, config_user):
@@ -447,28 +474,47 @@ def _update_fx_settings(settings, variable, config_user):
     # get fx variables either from user defined attribute or fixed
     def _get_fx_vars_from_attribute(step_settings, step_name):
         user_fx_vars = step_settings.get('fx_variables')
+        if isinstance(user_fx_vars, list):
+            user_fx_vars = _fx_list_to_dict(user_fx_vars)
+            step_settings['fx_variables'] = user_fx_vars
         if not user_fx_vars:
-            if step_name in ('mask_landsea', 'weighting_landsea_fraction'):
-                user_fx_vars = ['sftlf']
-                if variable['project'] != 'obs4mips':
-                    user_fx_vars.append('sftof')
-            elif step_name == 'mask_landseaice':
-                user_fx_vars = ['sftgif']
-            elif step_name in ('area_statistics', 'volume_statistics',
-                               'zonal_statistics'):
-                user_fx_vars = []
-        return user_fx_vars
+            default_fx = {
+                'area_statistics': {
+                    'areacella': None,
+                    'areacello': None,
+                },
+                'mask_landsea': {
+                    'sftlf': None,
+                },
+                'mask_landseaice': {
+                    'sftgif': None,
+                },
+                'volume_statistics': {
+                    'volcello': None,
+                },
+                'weighting_landsea_fraction': {
+                    'sftlf': None,
+                },
+            }
+            if variable['project'] != 'obs4mips':
+                default_fx['mask_landsea'].update({'sftof': None})
+                default_fx['weighting_landsea_fraction'].update(
+                    {'sftof': None})
+            step_settings['fx_variables'] = default_fx[step_name]
 
     fx_steps = [
         'mask_landsea', 'mask_landseaice', 'weighting_landsea_fraction',
-        'area_statistics', 'volume_statistics', 'zonal_statistics'
+        'area_statistics', 'volume_statistics'
     ]
-
-    for step_name, step_settings in settings.items():
+    for step_name in settings:
         if step_name in fx_steps:
-            fx_vars = _get_fx_vars_from_attribute(step_settings, step_name)
-            _update_fx_files(step_name, step_settings, variable, config_user,
-                             fx_vars)
+            _get_fx_vars_from_attribute(settings[step_name], step_name)
+            _update_fx_files(step_name, settings, variable, config_user,
+                             settings[step_name]['fx_variables'])
+            # Remove unused attribute in 'fx_steps' preprocessors.
+            # The fx_variables information is saved in
+            # the 'add_fx_variables' step.
+            settings[step_name].pop('fx_variables', None)
 
 
 def _read_attributes(filename):
@@ -509,9 +555,7 @@ def _get_ancestors(variable, config_user):
     logger.info("Using input files for variable %s of dataset %s:\n%s",
                 variable['short_name'], variable['dataset'],
                 '\n'.join(input_files))
-    if (not config_user.get('skip-nonexistent')
-            or variable['dataset'] == variable.get('reference_dataset')):
-        check.data_availability(input_files, variable, dirnames, filenames)
+    check.data_availability(input_files, variable, dirnames, filenames)
 
     # Set up provenance tracking
     for i, filename in enumerate(input_files):
@@ -650,6 +694,16 @@ def _match_products(products, variables):
     return grouped_products
 
 
+def _allow_skipping(ancestors, variable, config_user):
+    """Allow skipping of datasets."""
+    allow_skipping = all([
+        config_user.get('skip-nonexistent'),
+        not ancestors,
+        variable['dataset'] != variable.get('reference_dataset'),
+    ])
+    return allow_skipping
+
+
 def _get_preprocessor_products(variables, profile, order, ancestor_products,
                                config_user, name):
     """Get preprocessor product definitions for a set of datasets.
@@ -691,7 +745,7 @@ def _get_preprocessor_products(variables, profile, order, ancestor_products,
             try:
                 ancestors = _get_ancestors(variable, config_user)
             except RecipeError as ex:
-                if config_user.get('skip-nonexistent') and not ancestors:
+                if _allow_skipping(ancestors, variable, config_user):
                     logger.info("Skipping: %s", ex.message)
                 else:
                     missing_vars.add(ex.message)
@@ -919,6 +973,7 @@ class Recipe:
         self._cfg = deepcopy(config_user)
         self._cfg['write_ncl_interface'] = self._need_ncl(
             raw_recipe['diagnostics'])
+        self._raw_recipe = raw_recipe
         self._filename = os.path.basename(recipe_file)
         self._preprocessors = raw_recipe.get('preprocessors', {})
         if 'default' not in self._preprocessors:
@@ -952,9 +1007,8 @@ class Recipe:
     def _initialize_provenance(self, raw_documentation):
         """Initialize the recipe provenance."""
         doc = deepcopy(raw_documentation)
-        for key in doc:
-            if key in TAGS:
-                doc[key] = replace_tags(key, doc[key])
+
+        TAGS.replace_tags_in_dict(doc)
 
         return get_recipe_provenance(doc, self._filename)
 
@@ -968,12 +1022,13 @@ class Recipe:
         for name, raw_diagnostic in raw_diagnostics.items():
             diagnostic = {}
             diagnostic['name'] = name
+            additional_datasets = raw_diagnostic.get('additional_datasets', [])
+            datasets = (raw_datasets + additional_datasets)
             diagnostic['preprocessor_output'] = \
                 self._initialize_preprocessor_output(
                     name,
                     raw_diagnostic.get('variables', {}),
-                    raw_datasets +
-                    raw_diagnostic.get('additional_datasets', []))
+                    datasets)
             variable_names = tuple(raw_diagnostic.get('variables', {}))
             diagnostic['scripts'] = self._initialize_scripts(
                 name, raw_diagnostic.get('scripts'), variable_names)
@@ -996,37 +1051,38 @@ class Recipe:
         return datasets
 
     @staticmethod
-    def _expand_ensemble(variables):
-        """Expand ensemble members to multiple datasets.
+    def _expand_tag(variables, input_tag):
+        """Expand tags such as ensemble members or stardates to multiple
+        datasets.
 
         Expansion only supports ensembles defined as strings, not lists.
         """
         expanded = []
         regex = re.compile(r'\(\d+:\d+\)')
 
-        def expand_ensemble(variable):
-            ens = variable.get('ensemble', "")
-            match = regex.search(ens)
+        def expand_tag(variable, input_tag):
+            tag = variable.get(input_tag, "")
+            match = regex.search(tag)
             if match:
                 start, end = match.group(0)[1:-1].split(':')
                 for i in range(int(start), int(end) + 1):
                     expand = deepcopy(variable)
-                    expand['ensemble'] = regex.sub(str(i), ens, 1)
-                    expand_ensemble(expand)
+                    expand[input_tag] = regex.sub(str(i), tag, 1)
+                    expand_tag(expand, input_tag)
             else:
                 expanded.append(variable)
 
         for variable in variables:
-            ensemble = variable.get('ensemble', "")
-            if isinstance(ensemble, (list, tuple)):
-                for elem in ensemble:
+            tag = variable.get(input_tag, "")
+            if isinstance(tag, (list, tuple)):
+                for elem in tag:
                     if regex.search(elem):
                         raise RecipeError(
-                            f"In variable {variable}: ensemble expansion "
-                            "cannot be combined with ensemble lists")
+                            f"In variable {variable}: {input_tag} expansion "
+                            f"cannot be combined with {input_tag} lists")
                 expanded.append(variable)
             else:
-                expand_ensemble(variable)
+                expand_tag(variable, input_tag)
 
         return expanded
 
@@ -1065,6 +1121,7 @@ class Recipe:
         if 'fx' not in raw_variable.get('mip', ''):
             required_keys.update({'start_year', 'end_year'})
         for variable in variables:
+            _add_extra_facets(variable, self._cfg['extra_facets_dir'])
             if 'institute' not in variable:
                 institute = get_institutes(variable)
                 if institute:
@@ -1073,8 +1130,14 @@ class Recipe:
                 activity = get_activity(variable)
                 if activity:
                     variable['activity'] = activity
-            check.variable(variable, required_keys)
-        variables = self._expand_ensemble(variables)
+            if 'sub_experiment' in variable:
+                subexperiment_keys = deepcopy(required_keys)
+                subexperiment_keys.update({'sub_experiment'})
+                check.variable(variable, subexperiment_keys)
+            else:
+                check.variable(variable, required_keys)
+        variables = self._expand_tag(variables, 'ensemble')
+        variables = self._expand_tag(variables, 'sub_experiment')
         return variables
 
     def _initialize_preprocessor_output(self, diagnostic_name, raw_variables,
@@ -1326,9 +1389,9 @@ class Recipe:
                 tasks.add(task)
                 priority += 1
         if failed_tasks:
-            ex = RecipeError('Could not create all tasks')
-            ex.failed_tasks.extend(failed_tasks)
-            raise ex
+            recipe_error = RecipeError('Could not create all tasks')
+            recipe_error.failed_tasks.extend(failed_tasks)
+            raise recipe_error
         check.tasks_valid(tasks)
 
         # Resolve diagnostic ancestors
@@ -1370,7 +1433,7 @@ class Recipe:
 
         self.tasks.run(max_parallel_tasks=self._cfg['max_parallel_tasks'])
 
-    def get_product_output(self) -> dict:
+    def get_output(self) -> dict:
         """Return the paths to the output plots and data.
 
         Returns
@@ -1378,9 +1441,31 @@ class Recipe:
         product_filenames : dict
             Lists of products/attributes grouped by task.
         """
-        product_filenames = {}
+        output = {}
+
+        output['recipe_config'] = self._cfg
+        output['recipe_filename'] = self._filename
+        output['recipe_data'] = self._raw_recipe
+        output['task_output'] = {}
 
         for task in self.tasks:
-            product_filenames[task.name] = task.get_product_attributes()
+            output['task_output'][task.name] = task.get_product_attributes()
 
-        return product_filenames
+        return output
+
+    def write_html_summary(self):
+        """Write summary html file to the output dir."""
+        with warnings.catch_warnings():
+            # ignore import warnings
+            warnings.simplefilter("ignore")
+            # keep RecipeOutput here to avoid circular import
+            from esmvalcore.experimental.recipe_output import RecipeOutput
+            output = self.get_output()
+
+            try:
+                output = RecipeOutput.from_core_recipe_output(output)
+            except LookupError as error:
+                # See https://github.com/ESMValGroup/ESMValCore/issues/28
+                logging.debug("Could not write HTML report. %s", error)
+            else:
+                output.write_html()
