@@ -9,7 +9,8 @@ from shutil import which
 import yamale
 
 from ._data_finder import get_start_end_year
-from .preprocessor import PreprocessingTask, TIME_PREPROCESSORS
+from .preprocessor import TIME_PREPROCESSORS, PreprocessingTask
+from .preprocessor._multimodel import STATISTIC_MAPPING
 
 logger = logging.getLogger(__name__)
 
@@ -55,7 +56,7 @@ def recipe_with_schema(filename):
     logger.debug("Checking recipe against schema %s", schema_file)
     recipe = yamale.make_data(filename)
     schema = yamale.make_schema(schema_file)
-    yamale.validate(schema, recipe)
+    yamale.validate(schema, recipe, strict=False)
 
 
 def diagnostics(diags):
@@ -191,7 +192,7 @@ def extract_shape(settings):
 
 def valid_multimodel_statistic(statistic):
     """Check that `statistic` is a valid argument for multimodel stats."""
-    valid_names = ["mean", "median", "std", "min", "max"]
+    valid_names = ['std'] + list(STATISTIC_MAPPING.keys())
     valid_patterns = [r"^(p\d{1,2})(\.\d*)?$"]
     if not (statistic in valid_names
             or re.match(r'|'.join(valid_patterns), statistic)):
