@@ -1,4 +1,5 @@
 """Module for checking iris cubes against their CMOR definitions."""
+import datetime
 import logging
 from enum import IntEnum
 
@@ -11,8 +12,7 @@ import numpy as np
 
 from .table import CMOR_TABLES
 
-CheckLevels = IntEnum(
-    'CheckLevels', 'DEBUG STRICT DEFAULT RELAXED IGNORE')
+CheckLevels = IntEnum('CheckLevels', 'DEBUG STRICT DEFAULT RELAXED IGNORE')
 """Level of strictness of the checks.
 
    Attributes
@@ -25,6 +25,47 @@ CheckLevels = IntEnum(
 """
 
 
+def _get_next_month(month, year):
+    if month != 12:
+        return month + 1, year
+    return 1, year + 1
+
+
+def _get_time_bounds(time, freq):
+    bounds = []
+    for step, point in enumerate(time.points):
+        month = time.cell(step).point.month
+        year = time.cell(step).point.year
+        if freq in ['mon', 'mo']:
+            next_month, next_year = _get_next_month(month, year)
+            min_bound = time.units.date2num(
+                datetime.datetime(year, month, 1, 0, 0))
+            max_bound = time.units.date2num(
+                datetime.datetime(next_year, next_month, 1, 0, 0))
+        elif freq == 'yr':
+            min_bound = time.units.date2num(
+                datetime.datetime(year, 1, 1, 0, 0))
+            max_bound = time.units.date2num(
+                datetime.datetime(year + 1, 1, 1, 0, 0))
+        elif freq == 'dec':
+            min_bound = time.units.date2num(
+                datetime.datetime(year, 1, 1, 0, 0))
+            max_bound = time.units.date2num(
+                datetime.datetime(year + 10, 1, 1, 0, 0))
+        else:
+            delta = {
+                'day': 12 / 24,
+                '6hr': 3 / 24,
+                '3hr': 1.5 / 24,
+                '1hr': 0.5 / 24,
+            }
+            min_bound = point - delta[freq]
+            max_bound = point + delta[freq]
+        bounds.append([min_bound, max_bound])
+
+    return np.array(bounds)
+
+
 class CMORCheckError(Exception):
     """Exception raised when a cube does not pass the CMORCheck."""
 
@@ -102,8 +143,7 @@ class CMORCheck():
         return self._unstructured
 
     def check_metadata(self, logger=None):
-        """
-        Check the cube metadata.
+        """Check the cube metadata.
 
         Perform all the tests that do not require to have the data in memory.
 
@@ -125,7 +165,6 @@ class CMORCheck():
             If errors are found. If fail_on_error attribute is set to True,
             raises as soon as an error is detected. If set to False, it perform
             all checks and then raises.
-
         """
         if logger is not None:
             self._logger = logger
@@ -166,7 +205,6 @@ class CMORCheck():
             If errors are found. If fail_on_error attribute is set to True,
             raises as soon as an error is detected. If set to False, it perform
             all checks and then raises.
-
         """
         if logger is not None:
             self._logger = logger
@@ -189,10 +227,9 @@ class CMORCheck():
         ------
         CMORCheckError
             If any errors were reported before calling this method.
-
         """
         if self.has_errors():
-            msg = 'There were errors in variable {}:\n{}\nin cube:\n{}'
+            msg = 'There were errors in variable {}:\n{}\n in cube:\n{}'
             msg = msg.format(self._cube.var_name, '\n '.join(self._errors),
                              self._cube)
             raise CMORCheckError(msg)
@@ -204,7 +241,6 @@ class CMORCheck():
         ----------
         logger: logging.Logger
             Given logger
-
         """
         if self.has_warnings():
             msg = 'There were warnings in variable {}:\n{}\n'.format(
@@ -218,7 +254,6 @@ class CMORCheck():
         ----------
         logger: logging.Logger
             Given logger.
-
         """
         if self.has_debug_messages():
             msg = 'There were metadata changes in variable {}:\n{}\n'.format(
@@ -241,32 +276,27 @@ class CMORCheck():
                 if self.automatic_fixes:
                     self.report_warning(
                         'Standard name for {} changed from {} to {}',
-                        self._cube.var_name,
-                        self._cube.standard_name,
-                        self._cmor_var.standard_name
-                    )
+                        self._cube.var_name, self._cube.standard_name,
+                        self._cmor_var.standard_name)
                     self._cube.standard_name = self._cmor_var.standard_name
                 else:
-                    self.report_error(
-                        self._attr_msg, self._cube.var_name, 'standard_name',
-                        self._cmor_var.standard_name, self._cube.standard_name
-                    )
+                    self.report_error(self._attr_msg, self._cube.var_name,
+                                      'standard_name',
+                                      self._cmor_var.standard_name,
+                                      self._cube.standard_name)
         # Check long_name
         if self._cmor_var.long_name:
             if self._cube.long_name != self._cmor_var.long_name:
                 if self.automatic_fixes:
                     self.report_warning(
                         'Long name for {} changed from {} to {}',
-                        self._cube.var_name,
-                        self._cube.long_name,
-                        self._cmor_var.long_name
-                    )
+                        self._cube.var_name, self._cube.long_name,
+                        self._cmor_var.long_name)
                     self._cube.long_name = self._cmor_var.long_name
                 else:
-                    self.report_error(
-                        self._attr_msg, self._cube.var_name, 'long_name',
-                        self._cmor_var.long_name, self._cube.long_name
-                    )
+                    self.report_error(self._attr_msg, self._cube.var_name,
+                                      'long_name', self._cmor_var.long_name,
+                                      self._cube.long_name)
 
         # Check units
         if (self.automatic_fixes and self._cube.attributes.get(
@@ -278,10 +308,9 @@ class CMORCheck():
             units = self._get_effective_units()
             if self._cube.units != units:
                 if not self._cube.units.is_convertible(units):
-                    self.report_error(
-                        f'Variable {self._cube.var_name} units '
-                        f'{self._cube.units} can not be '
-                        f'converted to {self._cmor_var.units}')
+                    self.report_error(f'Variable {self._cube.var_name} units '
+                                      f'{self._cube.units} can not be '
+                                      f'converted to {self._cmor_var.units}')
                 else:
                     self.report_warning(
                         f'Variable {self._cube.var_name} units '
@@ -297,10 +326,9 @@ class CMORCheck():
                     self.report_warning('{}: attribute {} not present',
                                         self._cube.var_name, attr)
                 elif self._cube.attributes[attr] != attr_value:
-                    self.report_error(
-                        self._attr_msg, self._cube.var_name,
-                        attr, attr_value,
-                        self._cube.attributes[attr])
+                    self.report_error(self._attr_msg, self._cube.var_name,
+                                      attr, attr_value,
+                                      self._cube.attributes[attr])
 
     def _get_effective_units(self):
         """Get effective units."""
@@ -336,13 +364,13 @@ class CMORCheck():
         for coord in self._cube.coords():
             if coord.standard_name:
                 if coord.standard_name in standard_names:
-                    coords = [c.var_name for c in self._cube.coords(
-                        standard_name=coord.standard_name)
+                    coords = [
+                        c.var_name for c in self._cube.coords(
+                            standard_name=coord.standard_name)
                     ]
                     self.report_error(
                         'There are multiple coordinates with '
-                        f'standard_name "{coord.standard_name}": {coords}'
-                    )
+                        f'standard_name "{coord.standard_name}": {coords}')
                 else:
                     standard_names.add(coord.standard_name)
 
@@ -355,8 +383,8 @@ class CMORCheck():
             else:
                 try:
                     cube_coord = self._cube.coord(var_name=coordinate.out_name)
-                    if (cube_coord.standard_name is None and
-                            coordinate.standard_name == ''):
+                    if (cube_coord.standard_name is None
+                            and coordinate.standard_name == ''):
                         pass
                     elif cube_coord.standard_name != coordinate.standard_name:
                         self.report_critical(
@@ -394,11 +422,11 @@ class CMORCheck():
                         if coordinate.standard_name in ['time', 'latitude',
                                                         'longitude'] or \
                            coordinate.requested:
-                            self.report_critical(
-                                self._does_msg, coordinate.name, 'exist')
+                            self.report_critical(self._does_msg,
+                                                 coordinate.name, 'exist')
                         else:
-                            self.report_error(
-                                self._does_msg, coordinate.name, 'exist')
+                            self.report_error(self._does_msg, coordinate.name,
+                                              'exist')
 
     def _check_generic_level_dim_names(self, key, coordinate):
         standard_name = None
@@ -407,9 +435,7 @@ class CMORCheck():
         if coordinate.generic_lev_coords:
             for coord in coordinate.generic_lev_coords.values():
                 try:
-                    cube_coord = self._cube.coord(
-                        var_name=coord.out_name
-                    )
+                    cube_coord = self._cube.coord(var_name=coord.out_name)
                     out_name = coord.out_name
                     if cube_coord.standard_name == coord.standard_name:
                         standard_name = coord.standard_name
@@ -417,8 +443,7 @@ class CMORCheck():
                 except iris.exceptions.CoordinateNotFoundError:
                     try:
                         cube_coord = self._cube.coord(
-                            var_name=coord.standard_name
-                        )
+                            var_name=coord.standard_name)
                         standard_name = coord.standard_name
                         name = coord.name
                     except iris.exceptions.CoordinateNotFoundError:
@@ -427,29 +452,23 @@ class CMORCheck():
                 if not out_name:
                     self.report_error(
                         f'Generic level coordinate {key} '
-                        'has wrong var_name.',
-                    )
+                        'has wrong var_name.', )
                 level = coordinate.generic_lev_coords[name]
                 level.generic_level = True
                 level.generic_lev_coords = self._cmor_var.coordinates[
                     key].generic_lev_coords
                 self._cmor_var.coordinates[key] = level
-                self.report_debug_message(
-                    f'Generic level coordinate {key} '
-                    'will be checked against '
-                    f'{name} coordinate information'
-                )
+                self.report_debug_message(f'Generic level coordinate {key} '
+                                          'will be checked against '
+                                          f'{name} coordinate information')
             else:
                 if out_name:
                     self.report_critical(
                         f'Generic level coordinate {key} '
                         'has wrong standard_name '
-                        'or is not set.',
-                    )
+                        'or is not set.', )
                 else:
-                    self.report_critical(
-                        self._does_msg, key, 'exist'
-                    )
+                    self.report_critical(self._does_msg, key, 'exist')
 
     def _check_coords(self):
         """Check coordinates."""
@@ -508,9 +527,7 @@ class CMORCheck():
                 if not fixed:
                     self.report_critical(self._attr_msg, var_name, 'units',
                                          cmor.units, coord.units)
-        self._check_coord_values(cmor, coord, var_name)
-        self._check_coord_bounds(cmor, coord, var_name)
-        self._check_coord_monotonicity_and_direction(cmor, coord, var_name)
+        self._check_coord_points(cmor, coord, var_name)
 
     def _check_coord_bounds(self, cmor, coord, var_name):
         if cmor.must_have_bounds == 'yes' and not coord.has_bounds():
@@ -520,18 +537,30 @@ class CMORCheck():
                 except ValueError as ex:
                     self.report_warning(
                         'Can not guess bounds for coordinate {0} '
-                        'from var {1}: {2}', coord.var_name, var_name, ex
-                    )
+                        'from var {1}: {2}', coord.var_name, var_name, ex)
                 else:
                     self.report_warning(
                         'Added guessed bounds to coordinate {0} from var {1}',
-                        coord.var_name, var_name
-                    )
+                        coord.var_name, var_name)
             else:
                 self.report_warning(
                     'Coordinate {0} from var {1} does not have bounds',
-                    coord.var_name, var_name
-                )
+                    coord.var_name, var_name)
+
+    def _check_time_bounds(self, freq, time):
+        times = {'time', 'time1', 'time2', 'time3'}
+        key = times.intersection(self._cmor_var.coordinates)
+        cmor = self._cmor_var.coordinates[" ".join(key)]
+        if cmor.must_have_bounds == 'yes' and not time.has_bounds():
+            if self.automatic_fixes:
+                time.bounds = _get_time_bounds(time, freq)
+                self.report_warning(
+                    'Added guessed bounds to coordinate {0} from var {1}',
+                    time.var_name, self._cmor_var.short_name)
+            else:
+                self.report_warning(
+                    'Coordinate {0} from var {1} does not have bounds',
+                    time.var_name, self._cmor_var.short_name)
 
     def _check_coord_monotonicity_and_direction(self, cmor, coord, var_name):
         """Check monotonicity and direction of coordinate."""
@@ -557,15 +586,15 @@ class CMORCheck():
             if cmor.stored_direction == 'increasing':
                 if coord.points[0] > coord.points[1]:
                     if not self.automatic_fixes or coord.ndim > 1:
-                        self.report_critical(
-                            self._is_msg, var_name, 'increasing')
+                        self.report_critical(self._is_msg, var_name,
+                                             'increasing')
                     else:
                         self._reverse_coord(coord)
             elif cmor.stored_direction == 'decreasing':
                 if coord.points[0] < coord.points[1]:
                     if not self.automatic_fixes or coord.ndim > 1:
-                        self.report_critical(
-                            self._is_msg, var_name, 'decreasing')
+                        self.report_critical(self._is_msg, var_name,
+                                             'decreasing')
                     else:
                         self._reverse_coord(coord)
 
@@ -574,9 +603,19 @@ class CMORCheck():
         if coord.ndim == 1:
             self._cube = iris.util.reverse(self._cube,
                                            self._cube.coord_dims(coord))
+            reversed_coord = self._cube.coord(var_name=coord.var_name)
+            if reversed_coord.has_bounds():
+                bounds = reversed_coord.bounds
+                right_bounds = bounds[:-2, 1]
+                left_bounds = bounds[1:-1, 0]
+                if np.all(right_bounds != left_bounds):
+                    reversed_coord.bounds = np.fliplr(bounds)
+                    coord = reversed_coord
+            self.report_debug_message(f'Coordinate {coord.var_name} values'
+                                      'have been reversed.')
 
-    def _check_coord_values(self, coord_info, coord, var_name):
-        """Check coordinate values."""
+    def _check_coord_points(self, coord_info, coord, var_name):
+        """Check coordinate points: values, bounds and monotonicity."""
         # Check requested coordinate values exist in coord.points
         self._check_requested_values(coord, coord_info, var_name)
 
@@ -591,9 +630,9 @@ class CMORCheck():
                     l_fix_coord_value = self._check_longitude_min(
                         coord, var_name)
                 else:
-                    self.report_critical(
-                        self._vals_msg, var_name,
-                        '< {} ='.format('valid_min'), valid_min)
+                    self.report_critical(self._vals_msg, var_name,
+                                         '< {} ='.format('valid_min'),
+                                         valid_min)
 
         if coord_info.valid_max:
             valid_max = float(coord_info.valid_max)
@@ -603,14 +642,14 @@ class CMORCheck():
                     l_fix_coord_value = self._check_longitude_max(
                         coord, var_name)
                 else:
-                    self.report_critical(
-                        self._vals_msg, var_name,
-                        '> {} ='.format('valid_max'), valid_max)
+                    self.report_critical(self._vals_msg, var_name,
+                                         '> {} ='.format('valid_max'),
+                                         valid_max)
 
         if l_fix_coord_value:
             if coord.ndim == 1:
-                lon_extent = iris.coords.CoordExtent(
-                    coord, 0.0, 360., True, False)
+                lon_extent = iris.coords.CoordExtent(coord, 0.0, 360., True,
+                                                     False)
                 self._cube = self._cube.intersection(lon_extent)
             else:
                 new_lons = coord.points.copy()
@@ -624,20 +663,22 @@ class CMORCheck():
                 dims = self._cube.coord_dims(coord)
                 self._cube.remove_coord(coord)
                 self._cube.add_aux_coord(new_coord, dims)
+            coord = self._cube.coord(var_name=var_name)
+        self._check_coord_bounds(coord_info, coord, var_name)
+        self._check_coord_monotonicity_and_direction(coord_info, coord,
+                                                     var_name)
 
     def _check_longitude_max(self, coord, var_name):
         if np.any(coord.points > 720):
             self.report_critical(
-                f'{var_name} longitude coordinate has values > 720 degrees'
-            )
+                f'{var_name} longitude coordinate has values > 720 degrees')
             return False
         return True
 
     def _check_longitude_min(self, coord, var_name):
         if np.any(coord.points < -360):
             self.report_critical(
-                f'{var_name} longitude coordinate has values < -360 degrees'
-            )
+                f'{var_name} longitude coordinate has values < -360 degrees')
             return False
         return True
 
@@ -673,9 +714,8 @@ class CMORCheck():
 
         var_name = coord.var_name
         if not coord.is_monotonic():
-            self.report_error(
-                'Time coordinate for var {} is not monotonic', var_name
-            )
+            self.report_error('Time coordinate for var {} is not monotonic',
+                              var_name)
 
         if not coord.units.is_time_reference():
             self.report_critical(self._does_msg, var_name,
@@ -683,14 +723,11 @@ class CMORCheck():
         else:
             old_units = coord.units
             coord.convert_units(
-                cf_units.Unit(
-                    'days since 1850-1-1 00:00:00',
-                    calendar=coord.units.calendar))
+                cf_units.Unit('days since 1850-1-1 00:00:00',
+                              calendar=coord.units.calendar))
             simplified_cal = self._simplify_calendar(coord.units.calendar)
             coord.units = cf_units.Unit(coord.units.origin, simplified_cal)
-
             attrs = self._cube.attributes
-
             parent_time = 'parent_time_units'
             if parent_time in attrs:
                 if attrs[parent_time] in 'no parent':
@@ -750,6 +787,8 @@ class CMORCheck():
                 interval = intervals[freq]
                 target_interval = (interval[0] - tol, interval[1] + tol)
             elif freq.endswith('hr'):
+                if freq == 'hr':
+                    freq = '1hr'
                 frequency = freq[:-2]
                 if frequency == 'sub':
                     frequency = 1.0 / 24
@@ -768,7 +807,7 @@ class CMORCheck():
                     msg = '{}: Frequency {} does not match input data'
                     self.report_error(msg, var_name, freq)
                     break
-
+        self._check_time_bounds(freq, coord)
         # remove time_origin from attributes
         coord.attributes.pop('time_origin', None)
 
@@ -788,7 +827,6 @@ class CMORCheck():
         -------
         bool:
             True if there are pending errors, False otherwise.
-
         """
         return len(self._errors) > 0
 
@@ -799,7 +837,6 @@ class CMORCheck():
         -------
         bool:
             True if there are pending warnings, False otherwise.
-
         """
         return len(self._warnings) > 0
 
@@ -810,12 +847,11 @@ class CMORCheck():
         -------
         bool:
             True if there are pending debug messages, False otherwise.
-
         """
         return len(self._debug_messages) > 0
 
     def report(self, level, message, *args):
-        """Generic method to report a message from the checker
+        """Report a message from the checker.
 
         Parameters
         ----------
@@ -845,7 +881,8 @@ class CMORCheck():
                 self._warnings.append(msg)
         else:
             if self._failerr:
-                raise CMORCheckError(msg + '\nin cube:\n{}'.format(self._cube))
+                raise CMORCheckError(msg +
+                                     '\n in cube:\n{}'.format(self._cube))
             self._errors.append(msg)
 
     def report_critical(self, message, *args):
@@ -860,7 +897,6 @@ class CMORCheck():
             Message for the error.
         *args:
             arguments to format the message string.
-
         """
         self.report(CheckLevels.RELAXED, message, *args)
 
@@ -873,7 +909,6 @@ class CMORCheck():
             Message for the error.
         *args:
             arguments to format the message string.
-
         """
         self.report(CheckLevels.DEFAULT, message, *args)
 
@@ -886,7 +921,6 @@ class CMORCheck():
             Message for the warning.
         *args:
             arguments to format the message string.
-
         """
         self.report(CheckLevels.STRICT, message, *args)
 
@@ -899,7 +933,6 @@ class CMORCheck():
             Message for the debug logger.
         *args:
             arguments to format the message string
-
         """
         self.report(CheckLevels.DEBUG, message, *args)
 
@@ -924,19 +957,21 @@ def _get_cmor_checker(table,
         var_info = CMOR_TABLES['custom'].get_variable(mip, short_name)
 
     def _checker(cube):
-        return CMORCheck(
-            cube,
-            var_info,
-            frequency=frequency,
-            fail_on_error=fail_on_error,
-            check_level=check_level,
-            automatic_fixes=automatic_fixes)
+        return CMORCheck(cube,
+                         var_info,
+                         frequency=frequency,
+                         fail_on_error=fail_on_error,
+                         check_level=check_level,
+                         automatic_fixes=automatic_fixes)
 
     return _checker
 
 
-def cmor_check_metadata(cube, cmor_table, mip,
-                        short_name, frequency,
+def cmor_check_metadata(cube,
+                        cmor_table,
+                        mip,
+                        short_name,
+                        frequency,
                         check_level=CheckLevels.DEFAULT):
     """Check if metadata conforms to variable's CMOR definition.
 
@@ -946,26 +981,31 @@ def cmor_check_metadata(cube, cmor_table, mip,
     ----------
     cube: iris.cube.Cube
         Data cube to check.
-    cmor_table: basestring
+    cmor_table: str
         CMOR definitions to use.
     mip:
         Variable's mip.
-    short_name: basestring
+    short_name: str
         Variable's short name.
-    frequency: basestring
+    frequency: str
         Data frequency.
     check_level: CheckLevels
         Level of strictness of the checks.
-
     """
-    checker = _get_cmor_checker(cmor_table, mip,
-                                short_name, frequency,
+    checker = _get_cmor_checker(cmor_table,
+                                mip,
+                                short_name,
+                                frequency,
                                 check_level=check_level)
     checker(cube).check_metadata()
     return cube
 
 
-def cmor_check_data(cube, cmor_table, mip, short_name, frequency,
+def cmor_check_data(cube,
+                    cmor_table,
+                    mip,
+                    short_name,
+                    frequency,
                     check_level=CheckLevels.DEFAULT):
     """Check if data conforms to variable's CMOR definition.
 
@@ -975,19 +1015,21 @@ def cmor_check_data(cube, cmor_table, mip, short_name, frequency,
     ----------
     cube: iris.cube.Cube
         Data cube to check.
-    cmor_table: basestring
+    cmor_table: str
         CMOR definitions to use.
     mip:
         Variable's mip.
-    short_name: basestring
+    short_name: str
         Variable's short name
-    frequency: basestring
+    frequency: str
         Data frequency
     check_level: CheckLevels
         Level of strictness of the checks.
-
     """
-    checker = _get_cmor_checker(cmor_table, mip, short_name, frequency,
+    checker = _get_cmor_checker(cmor_table,
+                                mip,
+                                short_name,
+                                frequency,
                                 check_level=check_level)
     checker(cube).check_data()
     return cube
@@ -1003,20 +1045,27 @@ def cmor_check(cube, cmor_table, mip, short_name, frequency, check_level):
     ----------
     cube: iris.cube.Cube
         Data cube to check.
-    cmor_table: basestring
+    cmor_table: str
         CMOR definitions to use.
     mip:
         Variable's mip.
-    short_name: basestring
+    short_name: str
         Variable's short name.
-    frequency: basestring
+    frequency: str
         Data frequency.
     check_level: enum.IntEnum
         Level of strictness of the checks.
-
     """
-    cmor_check_metadata(cube, cmor_table, mip, short_name, frequency,
+    cmor_check_metadata(cube,
+                        cmor_table,
+                        mip,
+                        short_name,
+                        frequency,
                         check_level=check_level)
-    cmor_check_data(cube, cmor_table, mip, short_name, frequency,
+    cmor_check_data(cube,
+                    cmor_table,
+                    mip,
+                    short_name,
+                    frequency,
                     check_level=check_level)
     return cube
