@@ -10,6 +10,8 @@ import yaml
 from PIL import Image
 
 import esmvalcore
+from esmvalcore import __version__ as current_version
+from esmvalcore._config import TAGS
 from esmvalcore._recipe import TASKSEP, read_recipe_file
 from esmvalcore._recipe_checks import RecipeError
 from esmvalcore._task import DiagnosticTask
@@ -20,6 +22,34 @@ from esmvalcore.preprocessor._io import concatenate_callback
 from .test_diagnostic_run import write_config_user_file
 from .test_provenance import check_provenance
 
+TAGS_FOR_TESTING = {
+    'authors': {
+        'andela_bouwe': {
+            'name': 'Bouwe, Andela',
+        },
+    },
+    'projects': {
+        'c3s-magic': 'C3S MAGIC project',
+    },
+    'themes': {
+        'phys': 'physics',
+    },
+    'realms': {
+        'atmos': 'atmosphere',
+    },
+    'statistics': {
+        'mean': 'mean',
+        'var': 'variability',
+    },
+    'domains': {
+        'et': 'extra tropics',
+        'trop': 'tropics',
+    },
+    'plot_types': {
+        'zonal': 'zonal',
+    },
+}
+
 MANDATORY_DATASET_KEYS = (
     'dataset',
     'diagnostic',
@@ -47,6 +77,7 @@ MANDATORY_SCRIPT_SETTINGS_KEYS = (
 )
 
 DEFAULT_PREPROCESSOR_STEPS = (
+    'add_fx_variables',
     'cleanup',
     'cmor_check_data',
     'cmor_check_metadata',
@@ -56,6 +87,7 @@ DEFAULT_PREPROCESSOR_STEPS = (
     'fix_file',
     'fix_metadata',
     'load',
+    'remove_fx_variables',
     'save',
 )
 
@@ -84,35 +116,86 @@ def create_test_file(filename, tracking_id=None):
     iris.save(cube, filename)
 
 
-def _get_default_settings_for_chl(fix_dir, save_filename):
+def _get_default_settings_for_chl(fix_dir, save_filename, preprocessor):
     """Get default preprocessor settings for chl."""
+    standard_name = ('mass_concentration_of_phytoplankton_'
+                     'expressed_as_chlorophyll_in_sea_water')
     defaults = {
         'load': {
             'callback': concatenate_callback,
         },
         'concatenate': {},
         'fix_file': {
-            'project': 'CMIP5',
+            'alias': 'CanESM2',
             'dataset': 'CanESM2',
-            'short_name': 'chl',
+            'diagnostic': 'diagnostic_name',
+            'end_year': 2005,
+            'ensemble': 'r1i1p1',
+            'exp': 'historical',
+            'filename': fix_dir.replace('_fixed', '.nc'),
+            'frequency': 'yr',
+            'institute': ['CCCma'],
+            'long_name': 'Total Chlorophyll Mass Concentration',
             'mip': 'Oyr',
+            'modeling_realm': ['ocnBgchem'],
+            'original_short_name': 'chl',
             'output_dir': fix_dir,
+            'preprocessor': preprocessor,
+            'project': 'CMIP5',
+            'recipe_dataset_index': 0,
+            'short_name': 'chl',
+            'standard_name': standard_name,
+            'start_year': 2000,
+            'units': 'kg m-3',
+            'variable_group': 'chl',
         },
         'fix_data': {
             'check_level': CheckLevels.DEFAULT,
-            'project': 'CMIP5',
+            'alias': 'CanESM2',
             'dataset': 'CanESM2',
-            'short_name': 'chl',
-            'mip': 'Oyr',
+            'diagnostic': 'diagnostic_name',
+            'end_year': 2005,
+            'ensemble': 'r1i1p1',
+            'exp': 'historical',
+            'filename': fix_dir.replace('_fixed', '.nc'),
             'frequency': 'yr',
+            'institute': ['CCCma'],
+            'long_name': 'Total Chlorophyll Mass Concentration',
+            'mip': 'Oyr',
+            'modeling_realm': ['ocnBgchem'],
+            'original_short_name': 'chl',
+            'preprocessor': preprocessor,
+            'project': 'CMIP5',
+            'recipe_dataset_index': 0,
+            'short_name': 'chl',
+            'standard_name': standard_name,
+            'start_year': 2000,
+            'units': 'kg m-3',
+            'variable_group': 'chl',
         },
         'fix_metadata': {
             'check_level': CheckLevels.DEFAULT,
-            'project': 'CMIP5',
+            'alias': 'CanESM2',
             'dataset': 'CanESM2',
-            'short_name': 'chl',
-            'mip': 'Oyr',
+            'diagnostic': 'diagnostic_name',
+            'end_year': 2005,
+            'ensemble': 'r1i1p1',
+            'exp': 'historical',
+            'filename': fix_dir.replace('_fixed', '.nc'),
             'frequency': 'yr',
+            'institute': ['CCCma'],
+            'long_name': 'Total Chlorophyll Mass Concentration',
+            'mip': 'Oyr',
+            'modeling_realm': ['ocnBgchem'],
+            'original_short_name': 'chl',
+            'preprocessor': preprocessor,
+            'project': 'CMIP5',
+            'recipe_dataset_index': 0,
+            'short_name': 'chl',
+            'standard_name': standard_name,
+            'start_year': 2000,
+            'units': 'kg m-3',
+            'variable_group': 'chl',
         },
         'clip_start_end_year': {
             'start_year': 2000,
@@ -132,6 +215,11 @@ def _get_default_settings_for_chl(fix_dir, save_filename):
             'short_name': 'chl',
             'frequency': 'yr',
         },
+        'add_fx_variables': {
+            'fx_variables': {},
+            'check_level': CheckLevels.DEFAULT,
+        },
+        'remove_fx_variables': {},
         'cleanup': {
             'remove': [fix_dir]
         },
@@ -439,7 +527,8 @@ def test_default_preprocessor(tmp_path, patched_datafinder, config_user):
 
     fix_dir = os.path.join(
         preproc_dir, 'CMIP5_CanESM2_Oyr_historical_r1i1p1_chl_2000-2005_fixed')
-    defaults = _get_default_settings_for_chl(fix_dir, product.filename)
+    defaults = _get_default_settings_for_chl(fix_dir, product.filename,
+                                             'default')
     assert product.settings == defaults
 
 
@@ -479,7 +568,8 @@ def test_default_preprocessor_custom_order(tmp_path, patched_datafinder,
 
     fix_dir = os.path.join(
         preproc_dir, 'CMIP5_CanESM2_Oyr_historical_r1i1p1_chl_2000-2005_fixed')
-    defaults = _get_default_settings_for_chl(fix_dir, product.filename)
+    defaults = _get_default_settings_for_chl(fix_dir, product.filename,
+                                             'default_custom_order')
     assert product.settings == defaults
 
 
@@ -517,27 +607,70 @@ def test_default_fx_preprocessor(tmp_path, patched_datafinder, config_user):
         },
         'concatenate': {},
         'fix_file': {
-            'project': 'CMIP5',
+            'alias': 'CanESM2',
             'dataset': 'CanESM2',
-            'short_name': 'sftlf',
+            'diagnostic': 'diagnostic_name',
+            'ensemble': 'r0i0p0',
+            'exp': 'historical',
+            'filename': fix_dir.replace('_fixed', '.nc'),
+            'frequency': 'fx',
+            'institute': ['CCCma'],
+            'long_name': 'Land Area Fraction',
             'mip': 'fx',
+            'modeling_realm': ['atmos'],
+            'original_short_name': 'sftlf',
             'output_dir': fix_dir,
+            'preprocessor': 'default',
+            'project': 'CMIP5',
+            'recipe_dataset_index': 0,
+            'short_name': 'sftlf',
+            'standard_name': 'land_area_fraction',
+            'units': '%',
+            'variable_group': 'sftlf'
         },
         'fix_data': {
             'check_level': CheckLevels.DEFAULT,
-            'project': 'CMIP5',
+            'alias': 'CanESM2',
             'dataset': 'CanESM2',
-            'short_name': 'sftlf',
-            'mip': 'fx',
+            'diagnostic': 'diagnostic_name',
+            'ensemble': 'r0i0p0',
+            'exp': 'historical',
+            'filename': fix_dir.replace('_fixed', '.nc'),
             'frequency': 'fx',
+            'institute': ['CCCma'],
+            'long_name': 'Land Area Fraction',
+            'mip': 'fx',
+            'modeling_realm': ['atmos'],
+            'original_short_name': 'sftlf',
+            'preprocessor': 'default',
+            'project': 'CMIP5',
+            'recipe_dataset_index': 0,
+            'short_name': 'sftlf',
+            'standard_name': 'land_area_fraction',
+            'units': '%',
+            'variable_group': 'sftlf'
         },
         'fix_metadata': {
             'check_level': CheckLevels.DEFAULT,
-            'project': 'CMIP5',
+            'alias': 'CanESM2',
             'dataset': 'CanESM2',
-            'short_name': 'sftlf',
-            'mip': 'fx',
+            'diagnostic': 'diagnostic_name',
+            'ensemble': 'r0i0p0',
+            'exp': 'historical',
+            'filename': fix_dir.replace('_fixed', '.nc'),
             'frequency': 'fx',
+            'institute': ['CCCma'],
+            'long_name': 'Land Area Fraction',
+            'mip': 'fx',
+            'modeling_realm': ['atmos'],
+            'original_short_name': 'sftlf',
+            'preprocessor': 'default',
+            'project': 'CMIP5',
+            'recipe_dataset_index': 0,
+            'short_name': 'sftlf',
+            'standard_name': 'land_area_fraction',
+            'units': '%',
+            'variable_group': 'sftlf'
         },
         'cmor_check_metadata': {
             'check_level': CheckLevels.DEFAULT,
@@ -553,6 +686,11 @@ def test_default_fx_preprocessor(tmp_path, patched_datafinder, config_user):
             'short_name': 'sftlf',
             'frequency': 'fx',
         },
+        'add_fx_variables': {
+            'fx_variables': {},
+            'check_level': CheckLevels.DEFAULT,
+        },
+        'remove_fx_variables': {},
         'cleanup': {
             'remove': [fix_dir]
         },
@@ -1297,48 +1435,16 @@ def simulate_diagnostic_run(diagnostic_task):
     return record
 
 
-TAGS = {
-    'authors': {
-        'andela_bouwe': {
-            'name': 'Bouwe Andela',
-        },
-    },
-    'projects': {
-        'c3s-magic': 'C3S MAGIC project',
-    },
-    'themes': {
-        'phys': 'physics',
-    },
-    'realms': {
-        'atmos': 'atmosphere',
-    },
-    'statistics': {
-        'mean': 'mean',
-        'var': 'variability',
-    },
-    'domains': {
-        'et': 'extra tropics',
-        'trop': 'tropics',
-    },
-    'plot_types': {
-        'zonal': 'zonal',
-    },
-}
-
-
 def test_diagnostic_task_provenance(
     tmp_path,
     patched_datafinder,
-    monkeypatch,
     config_user,
 ):
-    monkeypatch.setattr(esmvalcore._config, 'TAGS', TAGS)
-    monkeypatch.setattr(esmvalcore._recipe, 'TAGS', TAGS)
-    monkeypatch.setattr(esmvalcore._task, 'TAGS', TAGS)
-
     script = tmp_path / 'diagnostic.py'
     script.write_text('')
 
+    TAGS.set_tag_values(TAGS_FOR_TESTING)
+
     content = dedent("""
         diagnostics:
           diagnostic_name:
@@ -1534,6 +1640,8 @@ def test_ensemble_expansion(tmp_path, patched_datafinder, config_user):
 
 
 def test_extract_shape(tmp_path, patched_datafinder, config_user):
+    TAGS.set_tag_values(TAGS_FOR_TESTING)
+
     content = dedent("""
         preprocessors:
           test:
@@ -1573,6 +1681,8 @@ def test_extract_shape(tmp_path, patched_datafinder, config_user):
                          ['shapefile', 'method', 'crop', 'decomposed'])
 def test_extract_shape_raises(tmp_path, patched_datafinder, config_user,
                               invalid_arg):
+    TAGS.set_tag_values(TAGS_FOR_TESTING)
+
     # Create shapefile
     shapefile = config_user['auxiliary_data_dir'] / Path('test.shp')
     shapefile.parent.mkdir(parents=True, exist_ok=True)
@@ -1617,6 +1727,8 @@ def test_extract_shape_raises(tmp_path, patched_datafinder, config_user,
 
 
 def test_weighting_landsea_fraction(tmp_path, patched_datafinder, config_user):
+    TAGS.set_tag_values(TAGS_FOR_TESTING)
+
     content = dedent("""
         preprocessors:
           landfrac_weighting:
@@ -1652,9 +1764,9 @@ def test_weighting_landsea_fraction(tmp_path, patched_datafinder, config_user):
     for product in task.products:
         assert 'weighting_landsea_fraction' in product.settings
         settings = product.settings['weighting_landsea_fraction']
-        assert len(settings) == 2
+        assert len(settings) == 1
         assert settings['area_type'] == 'land'
-        fx_variables = settings['fx_variables']
+        fx_variables = product.settings['add_fx_variables']['fx_variables']
         assert isinstance(fx_variables, dict)
         if product.attributes['project'] == 'obs4mips':
             assert len(fx_variables) == 1
@@ -1703,18 +1815,12 @@ def test_weighting_landsea_fraction_no_fx(tmp_path, patched_failing_datafinder,
     for product in task.products:
         assert 'weighting_landsea_fraction' in product.settings
         settings = product.settings['weighting_landsea_fraction']
-        assert len(settings) == 2
+        assert len(settings) == 1
         assert 'exclude' not in settings
         assert settings['area_type'] == 'land'
-        fx_variables = settings['fx_variables']
+        fx_variables = product.settings['add_fx_variables']['fx_variables']
         assert isinstance(fx_variables, dict)
-        if product.attributes['project'] == 'obs4mips':
-            assert len(fx_variables) == 1
-            assert fx_variables['sftlf'] == []
-        else:
-            assert len(fx_variables) == 2
-            assert fx_variables['sftlf'] == []
-            assert fx_variables['sftof'] == []
+        assert len(fx_variables) == 0
 
 
 def test_weighting_landsea_fraction_exclude(tmp_path, patched_datafinder,
@@ -1760,13 +1866,9 @@ def test_weighting_landsea_fraction_exclude(tmp_path, patched_datafinder,
             continue
         assert 'weighting_landsea_fraction' in product.settings
         settings = product.settings['weighting_landsea_fraction']
-        assert len(settings) == 2
+        assert len(settings) == 1
         assert 'exclude' not in settings
         assert settings['area_type'] == 'land'
-        fx_variables = settings['fx_variables']
-        assert isinstance(fx_variables, dict)
-        assert len(fx_variables) == 1
-        assert fx_variables.get('sftlf')
 
 
 def test_weighting_landsea_fraction_exclude_fail(tmp_path, patched_datafinder,
@@ -1840,9 +1942,9 @@ def test_landmask(tmp_path, patched_datafinder, config_user):
     for product in task.products:
         assert 'mask_landsea' in product.settings
         settings = product.settings['mask_landsea']
-        assert len(settings) == 2
+        assert len(settings) == 1
         assert settings['mask_out'] == 'sea'
-        fx_variables = settings['fx_variables']
+        fx_variables = product.settings['add_fx_variables']['fx_variables']
         assert isinstance(fx_variables, dict)
         fx_variables = fx_variables.values()
         if product.attributes['project'] == 'obs4mips':
@@ -1857,17 +1959,22 @@ def test_user_defined_fxvar(tmp_path, patched_datafinder, config_user):
           landmask:
             mask_landsea:
               mask_out: sea
-              fx_variables: [{'short_name': 'sftlf', 'exp': 'piControl'}]
+              fx_variables:
+                sftlf:
+                  exp: piControl
             mask_landseaice:
               mask_out: sea
-              fx_variables: [{'short_name': 'sftgif', 'exp': 'piControl'}]
+              fx_variables:
+                sftgif:
+                  exp: piControl
             volume_statistics:
               operator: mean
             area_statistics:
               operator: mean
-              fx_variables: [{'short_name': 'areacello', 'mip': 'fx',
-                         'exp': 'piControl'}]
-
+              fx_variables:
+                areacello:
+                  mip: fx
+                  exp: piControl
         diagnostics:
           diagnostic_name:
             variables:
@@ -1891,39 +1998,101 @@ def test_user_defined_fxvar(tmp_path, patched_datafinder, config_user):
 
     # landsea
     settings = product.settings['mask_landsea']
-    assert len(settings) == 2
+    assert len(settings) == 1
     assert settings['mask_out'] == 'sea'
-    fx_variables = settings['fx_variables']
+    fx_variables = product.settings['add_fx_variables']['fx_variables']
     assert isinstance(fx_variables, dict)
-    assert len(fx_variables) == 1
-    assert '_fx_' in fx_variables['sftlf']
-    assert '_piControl_' in fx_variables['sftlf']
+    assert len(fx_variables) == 4
+    assert '_fx_' in fx_variables['sftlf']['filename']
+    assert '_piControl_' in fx_variables['sftlf']['filename']
 
     # landseaice
     settings = product.settings['mask_landseaice']
-    assert len(settings) == 2
+    assert len(settings) == 1
     assert settings['mask_out'] == 'sea'
-    fx_variables = settings['fx_variables']
-    assert isinstance(fx_variables, dict)
-    assert len(fx_variables) == 1
-    assert '_fx_' in fx_variables['sftgif']
-    assert '_piControl_' in fx_variables['sftgif']
+    assert '_fx_' in fx_variables['sftlf']['filename']
+    assert '_piControl_' in fx_variables['sftlf']['filename']
 
     # volume statistics
     settings = product.settings['volume_statistics']
     assert len(settings) == 1
     assert settings['operator'] == 'mean'
-    assert 'fx_variables' not in settings
+    assert 'volcello' in fx_variables
 
     # area statistics
     settings = product.settings['area_statistics']
-    assert len(settings) == 2
+    assert len(settings) == 1
     assert settings['operator'] == 'mean'
-    fx_variables = settings['fx_variables']
+    assert '_fx_' in fx_variables['areacello']['filename']
+    assert '_piControl_' in fx_variables['areacello']['filename']
+
+
+def test_user_defined_fxlist(tmp_path, patched_datafinder, config_user):
+    content = dedent("""
+        preprocessors:
+          landmask:
+            mask_landsea:
+              mask_out: sea
+              fx_variables: [{'short_name': 'sftlf', 'exp': 'piControl'}]
+            mask_landseaice:
+              mask_out: sea
+              fx_variables: [{'short_name': 'sftgif', 'exp': 'piControl'}]
+            volume_statistics:
+              operator: mean
+            area_statistics:
+              operator: mean
+              fx_variables: [{'short_name': 'areacello', 'mip': 'fx',
+                         'exp': 'piControl'}]
+        diagnostics:
+          diagnostic_name:
+            variables:
+              gpp:
+                preprocessor: landmask
+                project: CMIP5
+                mip: Lmon
+                exp: historical
+                start_year: 2000
+                end_year: 2005
+                ensemble: r1i1p1
+                additional_datasets:
+                  - {dataset: CanESM2}
+            scripts: null
+        """)
+    recipe = get_recipe(tmp_path, content, config_user)
+
+    # Check custom fx variables
+    task = recipe.tasks.pop()
+    product = task.products.pop()
+
+    # landsea
+    settings = product.settings['mask_landsea']
+    assert len(settings) == 1
+    assert settings['mask_out'] == 'sea'
+    fx_variables = product.settings['add_fx_variables']['fx_variables']
     assert isinstance(fx_variables, dict)
-    assert len(fx_variables) == 1
-    assert '_fx_' in fx_variables['areacello']
-    assert '_piControl_' in fx_variables['areacello']
+    assert len(fx_variables) == 4
+    assert '_fx_' in fx_variables['sftlf']['filename']
+    assert '_piControl_' in fx_variables['sftlf']['filename']
+
+    # landseaice
+    settings = product.settings['mask_landseaice']
+    assert len(settings) == 1
+    assert settings['mask_out'] == 'sea'
+    assert '_fx_' in fx_variables['sftlf']['filename']
+    assert '_piControl_' in fx_variables['sftlf']['filename']
+
+    # volume statistics
+    settings = product.settings['volume_statistics']
+    assert len(settings) == 1
+    assert settings['operator'] == 'mean'
+    assert 'volcello' in fx_variables
+
+    # area statistics
+    settings = product.settings['area_statistics']
+    assert len(settings) == 1
+    assert settings['operator'] == 'mean'
+    assert '_fx_' in fx_variables['areacello']['filename']
+    assert '_piControl_' in fx_variables['areacello']['filename']
 
 
 def test_landmask_no_fx(tmp_path, patched_failing_datafinder, config_user):
@@ -1965,29 +2134,35 @@ def test_landmask_no_fx(tmp_path, patched_failing_datafinder, config_user):
     for product in task.products:
         assert 'mask_landsea' in product.settings
         settings = product.settings['mask_landsea']
-        assert len(settings) == 3
+        assert len(settings) == 2
         assert settings['mask_out'] == 'sea'
         assert settings['always_use_ne_mask'] is False
-        fx_variables = settings['fx_variables']
+        fx_variables = product.settings['add_fx_variables']['fx_variables']
         assert isinstance(fx_variables, dict)
         fx_variables = fx_variables.values()
         assert not any(fx_variables)
 
 
-def test_fx_vars_mip_change_cmip6(tmp_path, patched_datafinder, config_user):
+# TODO remove after fixing test below
+def test_fx_vars_mip_change_cmip6_incomplete(tmp_path,
+                                             patched_datafinder,
+                                             config_user):
+    TAGS.set_tag_values(TAGS_FOR_TESTING)
+
     content = dedent("""
         preprocessors:
           preproc:
            area_statistics:
              operator: mean
-             fx_variables: [
-               'areacella',
-               'areacello',
-               'clayfrac',
-               'sftlf',
-               'sftgif',
-               'sftof',
-             ]
+             fx_variables:
+               areacella:
+                 ensemble: r2i1p1f1
+               areacello:
+               clayfrac:
+               sftlf:
+               sftgif:
+                 mip: fx
+               sftof:
            mask_landsea:
              mask_out: sea
 
@@ -2019,44 +2194,239 @@ def test_fx_vars_mip_change_cmip6(tmp_path, patched_datafinder, config_user):
     # Check area_statistics
     assert 'area_statistics' in product.settings
     settings = product.settings['area_statistics']
-    assert len(settings) == 2
+    assert len(settings) == 1
     assert settings['operator'] == 'mean'
-    fx_variables = settings['fx_variables']
+    fx_variables = product.settings['add_fx_variables']['fx_variables']
     assert isinstance(fx_variables, dict)
     assert len(fx_variables) == 6
-    assert '_fx_' in fx_variables['areacella']
-    assert '_Ofx_' in fx_variables['areacello']
-    assert '_Efx_' in fx_variables['clayfrac']
-    assert '_fx_' in fx_variables['sftlf']
-    assert '_fx_' in fx_variables['sftgif']
-    assert '_Ofx_' in fx_variables['sftof']
+    assert '_fx_' in fx_variables['areacella']['filename']
+    assert '_r2i1p1f1_' in fx_variables['areacella']['filename']
+    assert '_Ofx_' in fx_variables['areacello']['filename']
+    assert '_Efx_' in fx_variables['clayfrac']['filename']
+    assert '_fx_' in fx_variables['sftlf']['filename']
+    assert '_Ofx_' in fx_variables['sftof']['filename']
 
     # Check mask_landsea
     assert 'mask_landsea' in product.settings
     settings = product.settings['mask_landsea']
-    assert len(settings) == 2
+    assert len(settings) == 1
     assert settings['mask_out'] == 'sea'
-    fx_variables = settings['fx_variables']
+
+
+@pytest.mark.skipif(current_version < "2.4.0",
+                    reason="github.com/ESMValGroup/ESMValCore/issues/1159")
+def test_fx_vars_mip_change_cmip6(tmp_path, patched_datafinder, config_user):
+    TAGS.set_tag_values(TAGS_FOR_TESTING)
+
+    content = dedent("""
+        preprocessors:
+          preproc:
+           area_statistics:
+             operator: mean
+             fx_variables:
+               areacella:
+                 ensemble: r2i1p1f1
+               areacello:
+               clayfrac:
+               sftlf:
+               sftgif:
+                 mip: fx
+               sftof:
+           mask_landsea:
+             mask_out: sea
+
+        diagnostics:
+          diagnostic_name:
+            variables:
+              tas:
+                preprocessor: preproc
+                project: CMIP6
+                mip: Amon
+                exp: historical
+                start_year: 2000
+                end_year: 2005
+                ensemble: r1i1p1f1
+                grid: gn
+                additional_datasets:
+                  - {dataset: CanESM5}
+            scripts: null
+        """)
+    recipe = get_recipe(tmp_path, content, config_user)
+
+    # Check generated tasks
+    assert len(recipe.tasks) == 1
+    task = recipe.tasks.pop()
+    assert task.name == 'diagnostic_name' + TASKSEP + 'tas'
+    assert len(task.products) == 1
+    product = task.products.pop()
+
+    # Check area_statistics
+    assert 'area_statistics' in product.settings
+    settings = product.settings['area_statistics']
+    assert len(settings) == 1
+    assert settings['operator'] == 'mean'
+    fx_variables = product.settings['add_fx_variables']['fx_variables']
     assert isinstance(fx_variables, dict)
-    fx_variables = fx_variables.values()
-    assert len(fx_variables) == 2
-    for fx_file in fx_variables:
-        if 'sftlf' in fx_file:
-            assert '_fx_' in fx_file
-        elif 'sftof' in fx_file:
-            assert '_Ofx_' in fx_file
-        else:
-            assert False
+    assert len(fx_variables) == 6
+    assert '_fx_' in fx_variables['areacella']['filename']
+    assert '_r2i1p1f1_' in fx_variables['areacella']['filename']
+    assert '_Ofx_' in fx_variables['areacello']['filename']
+    assert '_Efx_' in fx_variables['clayfrac']['filename']
+    assert '_fx_' in fx_variables['sftlf']['filename']
+    assert '_fx_' in fx_variables['sftgif']['filename']  # fails, see skipif
+    assert '_Ofx_' in fx_variables['sftof']['filename']
+
+    # Check mask_landsea
+    assert 'mask_landsea' in product.settings
+    settings = product.settings['mask_landsea']
+    assert len(settings) == 1
+    assert settings['mask_out'] == 'sea'
+
+
+# TODO remove when you fix test below
+def test_fx_list_mip_change_cmip6_incomplete(tmp_path,
+                                             patched_datafinder,
+                                             config_user):
+    content = dedent("""
+        preprocessors:
+          preproc:
+           area_statistics:
+             operator: mean
+             fx_variables: [
+               'areacella',
+               'areacello',
+               'clayfrac',
+               'sftlf',
+               'sftgif',
+               'sftof',
+               ]
+           mask_landsea:
+             mask_out: sea
+
+        diagnostics:
+          diagnostic_name:
+            variables:
+              tas:
+                preprocessor: preproc
+                project: CMIP6
+                mip: Amon
+                exp: historical
+                start_year: 2000
+                end_year: 2005
+                ensemble: r1i1p1f1
+                grid: gn
+                additional_datasets:
+                  - {dataset: CanESM5}
+            scripts: null
+        """)
+    recipe = get_recipe(tmp_path, content, config_user)
+
+    # Check generated tasks
+    assert len(recipe.tasks) == 1
+    task = recipe.tasks.pop()
+    assert task.name == 'diagnostic_name' + TASKSEP + 'tas'
+    assert len(task.products) == 1
+    product = task.products.pop()
+
+    # Check area_statistics
+    assert 'area_statistics' in product.settings
+    settings = product.settings['area_statistics']
+    assert len(settings) == 1
+    assert settings['operator'] == 'mean'
+    fx_variables = product.settings['add_fx_variables']['fx_variables']
+    assert isinstance(fx_variables, dict)
+    assert len(fx_variables) == 6
+    assert '_fx_' in fx_variables['areacella']['filename']
+    assert '_Ofx_' in fx_variables['areacello']['filename']
+    assert '_Efx_' in fx_variables['clayfrac']['filename']
+    assert '_fx_' in fx_variables['sftlf']['filename']
+    assert '_Ofx_' in fx_variables['sftof']['filename']
+
+    # Check mask_landsea
+    assert 'mask_landsea' in product.settings
+    settings = product.settings['mask_landsea']
+    assert len(settings) == 1
+    assert settings['mask_out'] == 'sea'
+
+
+@pytest.mark.skipif(current_version < "2.4.0",
+                    reason="github.com/ESMValGroup/ESMValCore/issues/1159")
+def test_fx_list_mip_change_cmip6(tmp_path, patched_datafinder, config_user):
+    content = dedent("""
+        preprocessors:
+          preproc:
+           area_statistics:
+             operator: mean
+             fx_variables: [
+               'areacella',
+               'areacello',
+               'clayfrac',
+               'sftlf',
+               'sftgif',
+               'sftof',
+               ]
+           mask_landsea:
+             mask_out: sea
+
+        diagnostics:
+          diagnostic_name:
+            variables:
+              tas:
+                preprocessor: preproc
+                project: CMIP6
+                mip: Amon
+                exp: historical
+                start_year: 2000
+                end_year: 2005
+                ensemble: r1i1p1f1
+                grid: gn
+                additional_datasets:
+                  - {dataset: CanESM5}
+            scripts: null
+        """)
+    recipe = get_recipe(tmp_path, content, config_user)
+
+    # Check generated tasks
+    assert len(recipe.tasks) == 1
+    task = recipe.tasks.pop()
+    assert task.name == 'diagnostic_name' + TASKSEP + 'tas'
+    assert len(task.products) == 1
+    product = task.products.pop()
+
+    # Check area_statistics
+    assert 'area_statistics' in product.settings
+    settings = product.settings['area_statistics']
+    assert len(settings) == 1
+    assert settings['operator'] == 'mean'
+    fx_variables = product.settings['add_fx_variables']['fx_variables']
+    assert isinstance(fx_variables, dict)
+    assert len(fx_variables) == 6
+    assert '_fx_' in fx_variables['areacella']['filename']
+    assert '_Ofx_' in fx_variables['areacello']['filename']
+    assert '_Efx_' in fx_variables['clayfrac']['filename']
+    assert '_fx_' in fx_variables['sftlf']['filename']
+    assert '_fx_' in fx_variables['sftgif']['filename']  # fails, see skipif
+    assert '_Ofx_' in fx_variables['sftof']['filename']
+
+    # Check mask_landsea
+    assert 'mask_landsea' in product.settings
+    settings = product.settings['mask_landsea']
+    assert len(settings) == 1
+    assert settings['mask_out'] == 'sea'
 
 
 def test_fx_vars_volcello_in_ofx_cmip6(tmp_path, patched_datafinder,
                                        config_user):
+    TAGS.set_tag_values(TAGS_FOR_TESTING)
+
     content = dedent("""
         preprocessors:
           preproc:
            volume_statistics:
              operator: mean
-             fx_variables: ['volcello']
+             fx_variables:
+               volcello:
+                 mip: Ofx
 
         diagnostics:
           diagnostic_name:
@@ -2086,13 +2456,13 @@ def test_fx_vars_volcello_in_ofx_cmip6(tmp_path, patched_datafinder,
     # Check volume_statistics
     assert 'volume_statistics' in product.settings
     settings = product.settings['volume_statistics']
-    assert len(settings) == 2
+    assert len(settings) == 1
     assert settings['operator'] == 'mean'
-    fx_variables = settings['fx_variables']
+    fx_variables = product.settings['add_fx_variables']['fx_variables']
     assert isinstance(fx_variables, dict)
     assert len(fx_variables) == 1
-    assert '_Omon_' in fx_variables['volcello']
-    assert '_Ofx_' not in fx_variables['volcello']
+    assert '_Omon_' not in fx_variables['volcello']['filename']
+    assert '_Ofx_' in fx_variables['volcello']['filename']
 
 
 def test_fx_dicts_volcello_in_ofx_cmip6(tmp_path, patched_datafinder,
@@ -2102,8 +2472,10 @@ def test_fx_dicts_volcello_in_ofx_cmip6(tmp_path, patched_datafinder,
           preproc:
            volume_statistics:
              operator: mean
-             fx_variables: [{'short_name': 'volcello', 'mip': 'Oyr',
-                         'exp': 'piControl'}]
+             fx_variables:
+               volcello:
+                 mip: Oyr
+                 exp: piControl
 
         diagnostics:
           diagnostic_name:
@@ -2133,14 +2505,14 @@ def test_fx_dicts_volcello_in_ofx_cmip6(tmp_path, patched_datafinder,
     # Check volume_statistics
     assert 'volume_statistics' in product.settings
     settings = product.settings['volume_statistics']
-    assert len(settings) == 2
+    assert len(settings) == 1
     assert settings['operator'] == 'mean'
-    fx_variables = settings['fx_variables']
+    fx_variables = product.settings['add_fx_variables']['fx_variables']
     assert isinstance(fx_variables, dict)
     assert len(fx_variables) == 1
-    assert '_Oyr_' in fx_variables['volcello']
-    assert '_piControl_' in fx_variables['volcello']
-    assert '_Omon_' not in fx_variables['volcello']
+    assert '_Oyr_' in fx_variables['volcello']['filename'][0]
+    assert '_piControl_' in fx_variables['volcello']['filename'][0]
+    assert '_Omon_' not in fx_variables['volcello']['filename'][0]
 
 
 def test_fx_vars_list_no_preproc_cmip6(tmp_path, patched_datafinder,
@@ -2192,7 +2564,8 @@ def test_fx_vars_list_no_preproc_cmip6(tmp_path, patched_datafinder,
     settings = product.settings['area_statistics']
     assert len(settings) == 1
     assert settings['operator'] == 'mean'
-    assert 'fx_variables' not in settings
+    fx_variables = product.settings['add_fx_variables']['fx_variables']
+    assert len(fx_variables) == 2
 
 
 def test_fx_vars_volcello_in_omon_cmip6(tmp_path, patched_failing_datafinder,
@@ -2202,7 +2575,9 @@ def test_fx_vars_volcello_in_omon_cmip6(tmp_path, patched_failing_datafinder,
           preproc:
            volume_statistics:
              operator: mean
-             fx_variables: ['volcello']
+             fx_variables:
+               volcello:
+                 mip: Omon
 
         diagnostics:
           diagnostic_name:
@@ -2232,13 +2607,13 @@ def test_fx_vars_volcello_in_omon_cmip6(tmp_path, patched_failing_datafinder,
     # Check volume_statistics
     assert 'volume_statistics' in product.settings
     settings = product.settings['volume_statistics']
-    assert len(settings) == 2
+    assert len(settings) == 1
     assert settings['operator'] == 'mean'
-    fx_variables = settings['fx_variables']
+    fx_variables = product.settings['add_fx_variables']['fx_variables']
     assert isinstance(fx_variables, dict)
     assert len(fx_variables) == 1
-    assert '_Ofx_' not in fx_variables['volcello']
-    assert '_Omon_' in fx_variables['volcello']
+    assert '_Ofx_' not in fx_variables['volcello']['filename'][0]
+    assert '_Omon_' in fx_variables['volcello']['filename'][0]
 
 
 def test_fx_vars_volcello_in_oyr_cmip6(tmp_path, patched_failing_datafinder,
@@ -2248,7 +2623,9 @@ def test_fx_vars_volcello_in_oyr_cmip6(tmp_path, patched_failing_datafinder,
           preproc:
            volume_statistics:
              operator: mean
-             fx_variables: ['volcello']
+             fx_variables:
+               volcello:
+                 mip: Oyr
 
         diagnostics:
           diagnostic_name:
@@ -2278,13 +2655,13 @@ def test_fx_vars_volcello_in_oyr_cmip6(tmp_path, patched_failing_datafinder,
     # Check volume_statistics
     assert 'volume_statistics' in product.settings
     settings = product.settings['volume_statistics']
-    assert len(settings) == 2
+    assert len(settings) == 1
     assert settings['operator'] == 'mean'
-    fx_variables = settings['fx_variables']
+    fx_variables = product.settings['add_fx_variables']['fx_variables']
     assert isinstance(fx_variables, dict)
     assert len(fx_variables) == 1
-    assert '_Ofx_' not in fx_variables['volcello']
-    assert '_Oyr_' in fx_variables['volcello']
+    assert '_Ofx_' not in fx_variables['volcello']['filename'][0]
+    assert '_Oyr_' in fx_variables['volcello']['filename'][0]
 
 
 def test_fx_vars_volcello_in_fx_cmip5(tmp_path, patched_datafinder,
@@ -2294,7 +2671,8 @@ def test_fx_vars_volcello_in_fx_cmip5(tmp_path, patched_datafinder,
           preproc:
            volume_statistics:
              operator: mean
-             fx_variables: ['volcello']
+             fx_variables:
+               volcello:
 
         diagnostics:
           diagnostic_name:
@@ -2323,13 +2701,13 @@ def test_fx_vars_volcello_in_fx_cmip5(tmp_path, patched_datafinder,
     # Check volume_statistics
     assert 'volume_statistics' in product.settings
     settings = product.settings['volume_statistics']
-    assert len(settings) == 2
+    assert len(settings) == 1
     assert settings['operator'] == 'mean'
-    fx_variables = settings['fx_variables']
+    fx_variables = product.settings['add_fx_variables']['fx_variables']
     assert isinstance(fx_variables, dict)
     assert len(fx_variables) == 1
-    assert '_fx_' in fx_variables['volcello']
-    assert '_Omon_' not in fx_variables['volcello']
+    assert '_fx_' in fx_variables['volcello']['filename']
+    assert '_Omon_' not in fx_variables['volcello']['filename']
 
 
 def test_wrong_project(tmp_path, patched_datafinder, config_user):
@@ -2338,7 +2716,8 @@ def test_wrong_project(tmp_path, patched_datafinder, config_user):
           preproc:
            volume_statistics:
              operator: mean
-             fx_variables: ['volcello']
+             fx_variables:
+               volcello:
 
         diagnostics:
           diagnostic_name:
@@ -2364,15 +2743,16 @@ def test_wrong_project(tmp_path, patched_datafinder, config_user):
 
 
 def test_invalid_fx_var_cmip6(tmp_path, patched_datafinder, config_user):
+    TAGS.set_tag_values(TAGS_FOR_TESTING)
+
     content = dedent("""
         preprocessors:
           preproc:
            area_statistics:
              operator: mean
-             fx_variables: [
-               'areacella',
-               'wrong_fx_variable',
-             ]
+             fx_variables:
+               areacella:
+               wrong_fx_variable:
 
         diagnostics:
           diagnostic_name:
@@ -2391,8 +2771,46 @@ def test_invalid_fx_var_cmip6(tmp_path, patched_datafinder, config_user):
             scripts: null
         """)
     msg = ("Requested fx variable 'wrong_fx_variable' not available in any "
-           "'fx'-related CMOR table")
+           "CMOR table")
     with pytest.raises(RecipeError) as rec_err_exp:
         get_recipe(tmp_path, content, config_user)
     assert str(rec_err_exp.value) == INITIALIZATION_ERROR_MSG
     assert msg in rec_err_exp.value.failed_tasks[0].message
+
+
+def test_multimodel_mask(tmp_path, patched_datafinder, config_user):
+    """Test ``mask_multimodel``."""
+    content = dedent("""
+        preprocessors:
+          preproc:
+            mask_multimodel:
+
+        diagnostics:
+          diagnostic_name:
+            variables:
+              tas:
+                preprocessor: preproc
+                project: CMIP5
+                mip: Amon
+                exp: historical
+                start_year: 2005
+                end_year: 2005
+                ensemble: r1i1p1
+                additional_datasets:
+                  - {dataset: BNU-ESM}
+                  - {dataset: CanESM2}
+                  - {dataset: HadGEM2-ES}
+            scripts: null
+        """)
+    recipe = get_recipe(tmp_path, content, config_user)
+
+    # Check generated tasks
+    assert len(recipe.tasks) == 1
+    task = recipe.tasks.pop()
+    assert task.name == f'diagnostic_name{TASKSEP}tas'
+
+    # Check mask_multimodel
+    assert len(task.products) == 3
+    for product in task.products:
+        assert 'mask_multimodel' in product.settings
+        assert product.settings['mask_multimodel'] == {}
