@@ -1,16 +1,23 @@
 """Unit tests for the :func:`esmvalcore.preprocessor._time` module."""
 
 import copy
+import datetime
 import unittest
+from typing import List, Tuple
 
 import iris
 import iris.coord_categorisation
 import iris.coords
+import iris.exceptions
 import numpy as np
 import pytest
 from cf_units import Unit
 from iris.cube import Cube
-from numpy.testing import assert_array_almost_equal, assert_array_equal
+from numpy.testing import (
+    assert_array_almost_equal,
+    assert_array_equal,
+    assert_raises,
+)
 
 import tests
 from esmvalcore.preprocessor._time import (
@@ -34,13 +41,13 @@ from esmvalcore.preprocessor._time import (
 )
 
 
-def _create_sample_cube():
+def _create_sample_cube(calendar='gregorian'):
     cube = Cube(np.arange(1, 25), var_name='co2', units='J')
     cube.add_dim_coord(
         iris.coords.DimCoord(
             np.arange(15., 720., 30.),
             standard_name='time',
-            units=Unit('days since 1950-01-01 00:00:00', calendar='gregorian'),
+            units=Unit('days since 1950-01-01 00:00:00', calendar=calendar),
         ),
         0,
     )
@@ -66,6 +73,12 @@ class TestExtractMonth(tests.Test):
         assert_array_equal(np.array([1, 1]),
                            sliced.coord('month_number').points)
 
+    def test_raises_if_extracted_cube_is_none(self):
+        """Test function for winter."""
+        sliced = extract_month(self.cube, 1)
+        with assert_raises(ValueError):
+            extract_month(sliced, 2)
+
     def test_get_january_with_existing_coord(self):
         """Test january extraction."""
         iris.coord_categorisation.add_month_number(self.cube, 'time')
@@ -87,6 +100,11 @@ class TestTimeSlice(tests.Test):
         """Prepare tests."""
         self.cube = _create_sample_cube()
 
+    def test_raises_if_extracted_cube_is_none(self):
+        """Test extract_time."""
+        with assert_raises(ValueError):
+            extract_time(self.cube, 2000, 1, 1, 2000, 12, 31)
+
     def test_extract_time(self):
         """Test extract_time."""
         sliced = extract_time(self.cube, 1950, 1, 1, 1950, 12, 31)
@@ -202,6 +220,16 @@ class TestExtractSeason(tests.Test):
         iris.coord_categorisation.add_month_number(sliced, 'time')
         assert_array_equal(np.array([1, 2, 12, 1, 2, 12]),
                            sliced.coord('month_number').points)
+        with assert_raises(iris.exceptions.CoordinateNotFoundError):
+            self.cube.coord('clim_season')
+        with assert_raises(iris.exceptions.CoordinateNotFoundError):
+            self.cube.coord('season_year')
+
+    def test_raises_if_extracted_cube_is_none(self):
+        """Test function for winter."""
+        sliced = extract_season(self.cube, 'DJF')
+        with assert_raises(ValueError):
+            extract_season(sliced, 'MAM')
 
     def test_get_djf_caps(self):
         """Test function works when season specified in caps."""
@@ -209,6 +237,10 @@ class TestExtractSeason(tests.Test):
         iris.coord_categorisation.add_month_number(sliced, 'time')
         assert_array_equal(np.array([1, 2, 12, 1, 2, 12]),
                            sliced.coord('month_number').points)
+        with assert_raises(iris.exceptions.CoordinateNotFoundError):
+            self.cube.coord('clim_season')
+        with assert_raises(iris.exceptions.CoordinateNotFoundError):
+            self.cube.coord('season_year')
 
     def test_get_mam(self):
         """Test function for spring."""
@@ -216,6 +248,10 @@ class TestExtractSeason(tests.Test):
         iris.coord_categorisation.add_month_number(sliced, 'time')
         assert_array_equal(np.array([3, 4, 5, 3, 4, 5]),
                            sliced.coord('month_number').points)
+        with assert_raises(iris.exceptions.CoordinateNotFoundError):
+            self.cube.coord('clim_season')
+        with assert_raises(iris.exceptions.CoordinateNotFoundError):
+            self.cube.coord('season_year')
 
     def test_get_jja(self):
         """Test function for summer."""
@@ -224,12 +260,25 @@ class TestExtractSeason(tests.Test):
         assert_array_equal(np.array([6, 7, 8, 6, 7, 8]),
                            sliced.coord('month_number').points)
 
+    def test_get_multiple_seasons(self):
+        """Test function for two seasons."""
+        sliced = [extract_season(self.cube, seas) for seas in ["JJA", "SON"]]
+        clim_coords = [sin_sli.coord("clim_season") for sin_sli in sliced]
+        assert_array_equal(clim_coords[0].points,
+                           ['JJA', 'JJA', 'JJA', 'JJA', 'JJA', 'JJA'])
+        assert_array_equal(clim_coords[1].points,
+                           ['SON', 'SON', 'SON', 'SON', 'SON', 'SON'])
+
     def test_get_son(self):
         """Test function for summer."""
         sliced = extract_season(self.cube, 'SON')
         iris.coord_categorisation.add_month_number(sliced, 'time')
         assert_array_equal(np.array([9, 10, 11, 9, 10, 11]),
                            sliced.coord('month_number').points)
+        with assert_raises(iris.exceptions.CoordinateNotFoundError):
+            self.cube.coord('clim_season')
+        with assert_raises(iris.exceptions.CoordinateNotFoundError):
+            self.cube.coord('season_year')
 
     def test_get_jf(self):
         """Test function for custom seasons."""
@@ -237,6 +286,10 @@ class TestExtractSeason(tests.Test):
         iris.coord_categorisation.add_month_number(sliced, 'time')
         assert_array_equal(np.array([1, 2, 1, 2]),
                            sliced.coord('month_number').points)
+        with assert_raises(iris.exceptions.CoordinateNotFoundError):
+            self.cube.coord('clim_season')
+        with assert_raises(iris.exceptions.CoordinateNotFoundError):
+            self.cube.coord('season_year')
 
 
 class TestClimatology(tests.Test):
@@ -764,6 +817,17 @@ class TestRegridTimeYearly(tests.Test):
         expected = self.cube_1.data
         diff_cube = newcube_2 - newcube_1
         assert_array_equal(diff_cube.data, expected)
+        # test bounds are set at [01-01-YEAR 00:00, 01-01-NEXT_YEAR 00:00]
+        timeunit_1 = newcube_1.coord('time').units
+        for i, time in enumerate(newcube_1.coord('time').points):
+            year_1 = timeunit_1.num2date(time).year
+            expected_minbound = timeunit_1.date2num(
+                datetime.datetime(year_1, 1, 1))
+            expected_maxbound = timeunit_1.date2num(
+                datetime.datetime(year_1 + 1, 1, 1))
+            assert_array_equal(
+                newcube_1.coord('time').bounds[i],
+                np.array([expected_minbound, expected_maxbound]))
 
 
 class TestRegridTimeMonthly(tests.Test):
@@ -800,6 +864,37 @@ class TestRegridTimeMonthly(tests.Test):
         expected = self.cube_1.data
         diff_cube = newcube_2 - newcube_1
         assert_array_equal(diff_cube.data, expected)
+        # test bounds are set at
+        # [01-MONTH-YEAR 00:00, 01-NEXT_MONTH-YEAR 00:00]
+        timeunit_1 = newcube_1.coord('time').units
+        for i, time in enumerate(newcube_1.coord('time').points):
+            month_1 = timeunit_1.num2date(time).month
+            year_1 = timeunit_1.num2date(time).year
+            next_month = month_1 + 1
+            next_year = year_1
+            if month_1 == 12:
+                next_month = 1
+                next_year += 1
+            expected_minbound = timeunit_1.date2num(
+                datetime.datetime(year_1, month_1, 1))
+            expected_maxbound = timeunit_1.date2num(
+                datetime.datetime(next_year, next_month, 1))
+            assert_array_equal(
+                newcube_1.coord('time').bounds[i],
+                np.array([expected_minbound, expected_maxbound]))
+
+    def test_regrid_time_different_calendar_bounds(self):
+        """Test bounds in different calendars."""
+        cube_360 = _create_sample_cube(calendar='360_day')
+        # Same cubes but differing time units
+        newcube_360 = regrid_time(cube_360, frequency='mon')
+        newcube_gregorian = regrid_time(self.cube_1, frequency='mon')
+        bounds_360 = newcube_360.coord('time').bounds
+        bounds_gregorian = newcube_gregorian.coord('time').bounds
+        # test value of the bounds is not the same
+        assert (bounds_360 != bounds_gregorian).any()
+        # assert length of the 360_day bounds interval is 30 days
+        assert (bounds_360[:, 1] - bounds_360[:, 0] == 30).all()
 
 
 class TestRegridTimeDaily(tests.Test):
@@ -846,6 +941,13 @@ class TestRegridTimeDaily(tests.Test):
         expected = self.cube_1.data
         diff_cube = newcube_2 - newcube_1
         self.assert_array_equal(diff_cube.data, expected)
+        # test bounds are set with a dt = 12/24 days
+        for i, time in enumerate(newcube_1.coord('time').points):
+            expected_minbound = time - 12 / 24
+            expected_maxbound = time + 12 / 24
+            assert_array_equal(
+                newcube_1.coord('time').bounds[i],
+                np.array([expected_minbound, expected_maxbound]))
 
 
 class TestRegridTime6Hourly(tests.Test):
@@ -892,6 +994,13 @@ class TestRegridTime6Hourly(tests.Test):
         expected = self.cube_1.data
         diff_cube = newcube_2 - newcube_1
         self.assert_array_equal(diff_cube.data, expected)
+        # test bounds are set with a dt = 3/24 days
+        for i, time in enumerate(newcube_1.coord('time').points):
+            expected_minbound = time - 3 / 24
+            expected_maxbound = time + 3 / 24
+            assert_array_equal(
+                newcube_1.coord('time').bounds[i],
+                np.array([expected_minbound, expected_maxbound]))
 
 
 class TestRegridTime3Hourly(tests.Test):
@@ -938,6 +1047,13 @@ class TestRegridTime3Hourly(tests.Test):
         expected = self.cube_1.data
         diff_cube = newcube_2 - newcube_1
         self.assert_array_equal(diff_cube.data, expected)
+        # test bounds are set with a dt = 1.5/24 days
+        for i, time in enumerate(newcube_1.coord('time').points):
+            expected_minbound = time - 1.5 / 24
+            expected_maxbound = time + 1.5 / 24
+            assert_array_equal(
+                newcube_1.coord('time').bounds[i],
+                np.array([expected_minbound, expected_maxbound]))
 
 
 class TestRegridTime1Hourly(tests.Test):
@@ -984,6 +1100,13 @@ class TestRegridTime1Hourly(tests.Test):
         expected = self.cube_1.data
         diff_cube = newcube_2 - newcube_1
         self.assert_array_equal(diff_cube.data, expected)
+        # test bounds are set with a dt = 0.5/24 days
+        for i, time in enumerate(newcube_1.coord('time').points):
+            expected_minbound = time - 0.5 / 24
+            expected_maxbound = time + 0.5 / 24
+            assert_array_equal(
+                newcube_1.coord('time').bounds[i],
+                np.array([expected_minbound, expected_maxbound]))
 
 
 class TestTimeseriesFilter(tests.Test):
@@ -1141,7 +1264,7 @@ def make_map_data(number_years=2):
     return cube
 
 
-PARAMETERS = []
+PARAMETERS: List[Tuple] = []
 for period in ('full', 'day', 'month', 'season'):
     PARAMETERS.append((period, None))
     if period == 'season':
